/*
 *  Engine/Entry.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;
using System.IO;
using System.Xml;
using Mono.Unix;


namespace Incollector.Engine {

	public class Entry {
		
		protected EntryStruct data;
		
		
		public int ID {
			get { return data.id; }
		}
		
		public int Version {
			get { return data.version; }
		}
		
		public string Category {
			get { return Engine.Text.PrepareText (data.data.Title, true); }
		}
		
		public Gtk.Widget PreviewWidget {
			get { return data.data.PreviewWidget; }
		}
		
		public DateTime Date {
			get { return data.date;  }
			set { data.date = value; }
		}
		
		public Incollector.DataTypes.DataType Data {
			get { return data.data;  }
			set { data.data = value; }
		}
		
		public int Rating {
			get { return data.rating;  }
			set { data.rating = value; }
		}
		
		public ArrayList Tags {
			get { return new ArrayList (data.tags); }
		}
		
		public string Comment {
			get { return data.comment;  }
			set { data.comment = value; }
		}
		
		public bool Imported {
			get { return data.imported;  }
			set { data.imported = value; }
		}
		
		
		
		public Entry ()
		{
			data = new EntryStruct ();
			data.tags = new ArrayList ();
			data.id = -1;
		}
		
		
		public Entry (EntryStruct entry)
		{
			data = entry.Clone ();
		}
		
		
		public Entry Clone ()
		{
			Entry ent = new Entry ();
			ent.data = this.data.Clone ();
			
			return ent;
		}
		
		
		public Entry (int id)
		{
			if (id != -1)
				data = DB.GetEntryByID (id);
			else {
				data = new EntryStruct ();
				data.tags = new ArrayList ();
				data.id = -1;
			}
		}
		
		
		public Entry (bool Copy, int id)
		{
			data = DB.GetEntryByID (id);
			data.tags = new ArrayList (data.tags);
			
			if (Copy == true)
				data.id = -1;
		}
		
		
		public void Save ()
		{
			if (data.id == -1)
				DB.AddEntry (data);
			else
				DB.ChangeEntry (data);
		}
		
		
		public bool Remove ()
		{
			if (data.id == -1)
				return false;
			
			DB.RemoveEntry (data);
			data.id = -1;
			
			return true;
		}
		
		
		public string GetDateString (bool DateFirstYear)
		{
			if (DateFirstYear == true)
				return Date.ToString ("yyyy-MM-dd");
			else
				return Date.ToString ("dd-MM-yyyy");
		}
		
		
		/// <summary>Adds a new tag to entry</summary>
		/// <param name="tag">Tag name</param>
		/// <returns>If tag exists, returns false</returns>
		public bool AddTag (string tag)
		{
			foreach (string str in data.tags)
				if (str == tag)
					return false;
			
			data.tags.Add (tag);
			RemoveTag (Catalog.GetString (EntryStruct.UNTAGGED));
			
			return true;
		}
		
		
		/// <summary>Removes tag</summary>
		/// <param name="tag">Tag name</param>
		public void RemoveTag (string tag)
		{
			int i = 0;
			foreach (string str in data.tags) {
				if (str == tag) {
					data.tags.RemoveAt (i);
					
					if (data.tags.Count == 0)
						data.tags.Add (Catalog.GetString (EntryStruct.UNTAGGED));
					
					return;
				}
				
				i++;
			}
			
			return;
		}
		
		
		/// <summary>If tag exists, returns true</summary>
		/// <param name="tag">Tag name</param>
		public bool TagExists (string tag)
		{
			foreach (string str in data.tags)
				if (str == tag)
					return true;
			
			return false;
		}
		
		
		/// <summary>Changes tag to new tag</summary>
		/// <param name="OldTag">Tag name which exists in tags list</param>
		/// <param name="NewTag">OldTag is replaced by this tag</param>
		public void ChangeTag (string OldTag, string NewTag)
		{
			for (int i = 0; i < data.tags.Count; i++)
				if ((string) data.tags [i] == OldTag) {
					data.tags [i] = NewTag;
					return;
				}
		}
		
		
		/// <summary>Removes all the tags.</summary>
		public void RemoveAllTags ()
		{
			data.tags.Clear ();
			data.tags.Add (Catalog.GetString (EntryStruct.UNTAGGED));
		}
		
		
		/// <summary>Returns tags list (separated by ', ')</summary>
		public string GetTagsString (bool ShowUntagged)
		{
			if (ShowUntagged == false)
				if (data.tags.Count == 1)
					if ((string) Tags [0] == Catalog.GetString (EntryStruct.UNTAGGED)) // if untagged, return empty string
						return "";
			
			string rtn = "";
			foreach (string str in data.tags)
				if (rtn.Length > 0)
					rtn += ", " + str;
				else
					rtn += str;
			
			return rtn;
		}
		
		
		/// <summary>Returns tags list (separated by ', ')</summary>
		public string GetTagsString ()
		{
			return GetTagsString (true);
		}
	}
}
